@@ -1,11 +1,10 @@
 import os
-from unittest.mock import patch
+from unittest.mock import PropertyMock, patch
 
 from pelican import readers
 from pelican.tests.support import get_settings, unittest
 from pelican.utils import SafeDatetime
 
-
 CUR_DIR = os.path.dirname(__file__)
 CONTENT_PATH = os.path.join(CUR_DIR, "content")
 
@@ -28,12 +27,24 @@ class ReaderTest(unittest.TestCase):
                 self.assertEqual(
                     value,
                     real_value,
-                    "Expected %s to have value %s, but was %s"
-                    % (key, value, real_value),
+                    f"Expected {key} to have value {value}, but was {real_value}",
                 )
             else:
                 self.fail(f"Expected {key} to have value {value}, but was not in Dict")
 
+    def test_markdown_disabled(self):
+        with patch.object(
+            readers.MarkdownReader, "enabled", new_callable=PropertyMock
+        ) as attr_mock:
+            attr_mock.return_value = False
+            readrs = readers.Readers(settings=get_settings())
+            self.assertEqual(
+                set(readers.MarkdownReader.file_extensions),
+                readrs.disabled_readers.keys(),
+            )
+            for val in readrs.disabled_readers.values():
+                self.assertEqual(readers.MarkdownReader, val.__class__)
+
 
 class TestAssertDictHasSubset(ReaderTest):
     def setUp(self):
@@ -398,15 +409,98 @@ class RstReaderTest(ReaderTest):
         except ImportError:
             return unittest.skip("need the typogrify distribution")
 
+    def test_typogrify_ignore_filters(self):
+        try:
+            # typogrify should be able to ignore user specified filters.
+            page = self.read_file(
+                path="article_with_code_block.rst",
+                TYPOGRIFY=True,
+                TYPOGRIFY_OMIT_FILTERS=["amp"],
+            )
+            expected = (
+                "<p>An article with some&nbsp;code</p>\n"
+                '<div class="highlight"><pre><span></span>'
+                '<span class="n">x</span>'
+                ' <span class="o">&amp;</span>'
+                ' <span class="n">y</span>\n</pre></div>\n'
+                "<p>A block&nbsp;quote:</p>\n<blockquote>\nx "
+                "&amp; y</blockquote>\n"
+                "<p>Normal:\nx &amp;&nbsp;y</p>\n"
+            )
+            self.assertEqual(page.content, expected)
+
+            page = self.read_file(
+                path="article.rst",
+                TYPOGRIFY=True,
+                TYPOGRIFY_OMIT_FILTERS=["smartypants"],
+            )
+            expected = (
+                '<p><span class="caps">THIS</span> is some content. '
+                "With some stuff to&nbsp;&quot;typogrify&quot;...</p>\n"
+                '<p>Now with added support for <abbr title="three letter '
+                'acronym"><span class="caps">TLA</span></abbr>.</p>\n'
+            )
+            self.assertEqual(page.content, expected)
+
+            page = self.read_file(
+                path="article.rst",
+                TYPOGRIFY=True,
+                TYPOGRIFY_OMIT_FILTERS=["caps"],
+            )
+            expected = (
+                "<p>THIS is some content. "
+                "With some stuff to&nbsp;&#8220;typogrify&#8221;&#8230;</p>\n"
+                '<p>Now with added support for <abbr title="three letter '
+                'acronym">TLA</abbr>.</p>\n'
+            )
+            self.assertEqual(page.content, expected)
+
+            page = self.read_file(
+                path="article.rst",
+                TYPOGRIFY=True,
+                TYPOGRIFY_OMIT_FILTERS=["initial_quotes"],
+            )
+            expected = (
+                '<p><span class="caps">THIS</span> is some content. '
+                "With some stuff to&nbsp;&#8220;typogrify&#8221;&#8230;</p>\n"
+                '<p>Now with added support for <abbr title="three letter '
+                'acronym"><span class="caps">TLA</span></abbr>.</p>\n'
+            )
+            self.assertEqual(page.content, expected)
+
+            page = self.read_file(
+                path="article.rst",
+                TYPOGRIFY=True,
+                TYPOGRIFY_OMIT_FILTERS=["widont"],
+            )
+            expected = (
+                '<p><span class="caps">THIS</span> is some content. '
+                "With some stuff to "
+                "&#8220;typogrify&#8221;&#8230;</p>\n<p>Now with added "
+                'support for <abbr title="three letter acronym">'
+                '<span class="caps">TLA</span></abbr>.</p>\n'
+            )
+            self.assertEqual(page.content, expected)
+
+            page = self.read_file(
+                path="article.rst",
+                TYPOGRIFY=True,
+                TYPOGRIFY_OMIT_FILTERS=["this-filter-does-not-exists"],
+            )
+            self.assertRaises(TypeError)
+        except ImportError:
+            return unittest.skip("need the typogrify distribution")
+        except TypeError:
+            return unittest.skip("need typogrify version 2.1.0 or later")
+
     def test_typogrify_ignore_tags(self):
         try:
-            # typogrify should be able to ignore user specified tags,
-            # but tries to be clever with widont extension
+            # typogrify should be able to ignore user specified tags.
             page = self.read_file(
                 path="article.rst", TYPOGRIFY=True, TYPOGRIFY_IGNORE_TAGS=["p"]
             )
             expected = (
-                "<p>THIS is some content. With some stuff to&nbsp;"
+                "<p>THIS is some content. With some stuff to "
                 "&quot;typogrify&quot;...</p>\n<p>Now with added "
                 'support for <abbr title="three letter acronym">'
                 "TLA</abbr>.</p>\n"
@@ -592,7 +686,7 @@ class MdReaderTest(ReaderTest):
             "modified": SafeDatetime(2012, 11, 1),
             "multiline": [
                 "Line Metadata should be handle properly.",
-                "See syntax of Meta-Data extension of " "Python Markdown package:",
+                "See syntax of Meta-Data extension of Python Markdown package:",
                 "If a line is indented by 4 or more spaces,",
                 "that line is assumed to be an additional line of the value",
                 "for the previous keyword.",
