@@ -6,6 +6,8 @@ from datetime import timezone
 from sys import platform
 from tempfile import mkdtemp
 
+import watchfiles
+
 try:
     from zoneinfo import ZoneInfo
 except ModuleNotFoundError:
@@ -13,7 +15,7 @@ except ModuleNotFoundError:
 
 from pelican import utils
 from pelican.generators import TemplatePagesGenerator
-from pelican.settings import read_settings
+from pelican.settings import DEFAULT_CONFIG, read_settings
 from pelican.tests.support import (
     LoggedTestCase,
     get_article,
@@ -23,9 +25,17 @@ from pelican.tests.support import (
 from pelican.writers import Writer
 
 
-class TestUtils(LoggedTestCase):
+class ClassDeprAttr:
     _new_attribute = "new_value"
 
+    @utils.deprecated_attribute(
+        old="_old_attribute", new="_new_attribute", since=(3, 1, 0), remove=(4, 1, 3)
+    )
+    def _old_attribute():
+        return None
+
+
+class TestUtils(LoggedTestCase):
     def setUp(self):
         super().setUp()
         self.temp_output = mkdtemp(prefix="pelicantests.")
@@ -34,15 +44,10 @@ class TestUtils(LoggedTestCase):
         super().tearDown()
         shutil.rmtree(self.temp_output)
 
-    @utils.deprecated_attribute(
-        old="_old_attribute", new="_new_attribute", since=(3, 1, 0), remove=(4, 1, 3)
-    )
-    def _old_attribute():
-        return None
-
     def test_deprecated_attribute(self):
-        value = self._old_attribute
-        self.assertEqual(value, self._new_attribute)
+        test_class = ClassDeprAttr()
+        value = test_class._old_attribute
+        self.assertEqual(value, test_class._new_attribute)
         self.assertLogCountEqual(
             count=1,
             msg=(
@@ -207,13 +212,10 @@ class TestUtils(LoggedTestCase):
             )
 
         # check with preserve case
-        for value, expected in samples:
-            self.assertEqual(
-                utils.slugify(
-                    "Çığ", regex_subs=subs, preserve_case=True, use_unicode=True
-                ),
-                "Çığ",
-            )
+        self.assertEqual(
+            utils.slugify("Çığ", regex_subs=subs, preserve_case=True, use_unicode=True),
+            "Çığ",
+        )
 
         # check normalization
         samples = (
@@ -401,6 +403,23 @@ class TestUtils(LoggedTestCase):
         self.assertEqual(utils.truncate_html_words("&#1234 text", 20), "&#1234 text")
         self.assertEqual(utils.truncate_html_words("&#xabc text", 20), "&#xabc text")
 
+    def test_truncate_html_paragraphs(self):
+        one = "<p>one</p>"
+
+        self.assertEqual(utils.truncate_html_paragraphs(one, 0), "")
+        self.assertEqual(utils.truncate_html_paragraphs(one, 1), one)
+        self.assertEqual(utils.truncate_html_paragraphs(one, 2), one)
+
+        two = one + "<p>two</p>"
+        self.assertEqual(utils.truncate_html_paragraphs(two, 1), one)
+        self.assertEqual(utils.truncate_html_paragraphs(two, 2), two)
+
+        three = two + "<p>three</p>"
+        self.assertEqual(utils.truncate_html_paragraphs(three, 1), one)
+        self.assertEqual(utils.truncate_html_paragraphs(three, 2), two)
+        self.assertEqual(utils.truncate_html_paragraphs(three, 3), three)
+        self.assertEqual(utils.truncate_html_paragraphs(three, 4), three)
+
     def test_process_translations(self):
         fr_articles = []
         en_articles = []
@@ -721,11 +740,11 @@ class TestCopy(unittest.TestCase):
 
     def _exist_file(self, *path):
         path = os.path.join(self.root_dir, *path)
-        self.assertTrue(os.path.isfile(path), "File does not exist: %s" % path)
+        self.assertTrue(os.path.isfile(path), f"File does not exist: {path}")
 
     def _exist_dir(self, *path):
         path = os.path.join(self.root_dir, *path)
-        self.assertTrue(os.path.exists(path), "Directory does not exist: %s" % path)
+        self.assertTrue(os.path.exists(path), f"Directory does not exist: {path}")
 
     def test_copy_file_same_path(self):
         self._create_file("a.txt")
@@ -919,14 +938,14 @@ class TestSanitisedJoin(unittest.TestCase):
     def test_detect_parent_breakout(self):
         with self.assertRaisesRegex(
             RuntimeError,
-            "Attempted to break out of output directory to " "(.*?:)?/foo/test",
+            "Attempted to break out of output directory to (.*?:)?/foo/test",
         ):  # (.*?:)? accounts for Windows root
             utils.sanitised_join("/foo/bar", "../test")
 
     def test_detect_root_breakout(self):
         with self.assertRaisesRegex(
             RuntimeError,
-            "Attempted to break out of output directory to " "(.*?:)?/test",
+            "Attempted to break out of output directory to (.*?:)?/test",
         ):  # (.*?:)? accounts for Windows root
             utils.sanitised_join("/foo/bar", "/test")
 
@@ -966,3 +985,75 @@ class TestMemoized(unittest.TestCase):
             container.get.cache.clear()
             self.assertEqual("bar", container.get("bar"))
             get_mock.assert_called_once_with("bar")
+
+
+class TestStringUtils(unittest.TestCase):
+    def test_file_suffix(self):
+        self.assertEqual("", utils.file_suffix(""))
+        self.assertEqual("", utils.file_suffix("foo"))
+        self.assertEqual("md", utils.file_suffix("foo.md"))
+
+
+class TestFileChangeFilter(unittest.TestCase):
+    ignore_file_patterns = DEFAULT_CONFIG["IGNORE_FILES"]
+
+    def test_regular_files_not_filtered(self):
+        filter = utils.FileChangeFilter(ignore_file_patterns=self.ignore_file_patterns)
+        basename = "article.rst"
+        full_path = os.path.join(os.path.dirname(__file__), "content", basename)
+
+        for change in watchfiles.Change:
+            self.assertTrue(filter(change=change, path=basename))
+            self.assertTrue(filter(change=change, path=full_path))
+
+    def test_dotfiles_filtered(self):
+        filter = utils.FileChangeFilter(ignore_file_patterns=self.ignore_file_patterns)
+        basename = ".config"
+        full_path = os.path.join(os.path.dirname(__file__), "content", basename)
+
+        # Testing with just the hidden file name and the full file path to the hidden file
+        for change in watchfiles.Change:
+            self.assertFalse(filter(change=change, path=basename))
+            self.assertFalse(filter(change=change, path=full_path))
+
+    def test_default_filters(self):
+        # Testing a subset of the default filters
+        # For reference: https://watchfiles.helpmanual.io/api/filters/#watchfiles.DefaultFilter.ignore_dirs
+        filter = utils.FileChangeFilter(ignore_file_patterns=[])
+        test_basenames = [
+            "__pycache__",
+            ".git",
+            ".hg",
+            ".svn",
+            ".tox",
+            ".venv",
+            ".idea",
+            "node_modules",
+            ".mypy_cache",
+            ".pytest_cache",
+            ".hypothesis",
+            ".DS_Store",
+            "flycheck_file",
+            "test_file~",
+        ]
+
+        for basename in test_basenames:
+            full_path = os.path.join(os.path.dirname(__file__), basename)
+            for change in watchfiles.Change:
+                self.assertFalse(filter(change=change, path=basename))
+                self.assertFalse(filter(change=change, path=full_path))
+
+    def test_custom_ignore_pattern(self):
+        filter = utils.FileChangeFilter(ignore_file_patterns=["*.rst"])
+        basename = "article.rst"
+        full_path = os.path.join(os.path.dirname(__file__), basename)
+        for change in watchfiles.Change:
+            self.assertFalse(filter(change=change, path=basename))
+            self.assertFalse(filter(change=change, path=full_path))
+
+        # If the user changes `IGNORE_FILES` to only contain ['*.rst'], then dotfiles would not be filtered anymore
+        basename = ".config"
+        full_path = os.path.join(os.path.dirname(__file__), basename)
+        for change in watchfiles.Change:
+            self.assertTrue(filter(change=change, path=basename))
+            self.assertTrue(filter(change=change, path=full_path))
